from unittest.mock import patch
from tuxbake.models import OEBuild


def test_build(oebuild_repo_init_object):
    from tuxbake.build import build

    oebuild_repo_init_object.local_manifest = None
    oebuild_object = oebuild_repo_init_object.as_dict()

    with patch.object(OEBuild, "do_build", return_value="build_called"):
        data = build(**oebuild_object)
        assert data == OEBuild(**oebuild_object)
        ret_val = data.do_build()
        assert ret_val == "build_called"



import pytest
from tuxmake.runtime import Runtime
from tuxbake.exceptions import TuxbakeRunCmdError, TuxbakeParsingError
from pathlib import Path
import os
from tuxbake.models import OEBuild
from dataclasses import asdict


def test_oebuild(oebuild_git_object):
    (
        oebuild,
        git_repo_1,
        git_repo_2,
        git_branch_1,
        git_branch_2,
        git_sha_1,
        git_sha_2,
        source_dir,
    ) = oebuild_git_object
    assert oebuild.git_trees[0].url == git_repo_1
    assert oebuild.git_trees[0].branch == git_branch_1
    assert oebuild.git_trees[1].url == git_repo_2
    assert oebuild.git_trees[1].branch == git_branch_2
    assert oebuild.git_trees[0].sha == git_sha_1
    assert oebuild.git_trees[1].sha == git_sha_2
    assert oebuild.src_dir == source_dir


def test_repo_init_oebuild(oebuild_repo_init_object):
    oebuild = oebuild_repo_init_object
    assert oebuild.repo.url == "https://gitlab.com/alok.ranjan1/test-project"
    assert oebuild.repo.branch == "main"
    assert oebuild.repo.manifest == "default.xml"


def test_do_cleanup(oebuild_repo_init_object):
    oebuild_obj = oebuild_repo_init_object
    oebuild_obj._runtime = Runtime.get(None)
    oebuild_obj.do_cleanup()


""" Test map for validation of all api arguments. can add or delete ..depending upon use case"""
test_map = {
    "envsetup": [None, 1],
    "target": [None, 1],
    "distro": [1],
    "repo": [
        {"url": "", "branch": "main", "manifest": "default.xml"},
        {"url": 1, "branch": "main", "manifest": "default.xml"},
        {"url": "file://", "branch": "main", "manifest": "default.xml"},
        {"url": "git://" + "a" * 1020, "branch": "main", "manifest": "default.xml"},
        {
            "url": "https://gitlab.com/alok.ranjan1/test-project",
            "branch": None,
            "manifest": "default.xml",
        },
        {
            "url": "https://gitlab.com/alok.ranjan1/test-project",
            "branch": "main",
            "manifest": "default.txt",
        },
    ],
    "git_trees": [
        [{"url": "", "branch": "main", "ref": None, "sha": None}],
        [{"url": 1, "branch": "main", "ref": None, "sha": None}],
        [{"url": "file://", "branch": "main", "ref": None, "sha": None}],
        [{"url": "git://" + "a" * 1020, "branch": "main", "ref": None, "sha": None}],
        [
            {
                "url": "https://gitlab.com/alok.ranjan1/test-project",
                "branch": "main",
                "ref": 1,
                "sha": None,
            }
        ],
        [
            {
                "url": "https://gitlab.com/alok.ranjan1/test-project",
                "branch": "main",
                "ref": None,
                "sha": 1,
            }
        ],
        [
            {
                "url": "https://gitlab.com/alok.ranjan1/test-project",
                "branch": "main",
                "ref": "a" * 129,
                "sha": None,
            }
        ],
        [
            {
                "url": "https://gitlab.com/alok.ranjan1/test-project",
                "branch": "main",
                "ref": "?test",
                "sha": None,
            }
        ],
        [
            {
                "url": "https://gitlab.com/alok.ranjan1/test-project",
                "branch": "main",
                "ref": None,
                "sha": "test",
            }
        ],
    ],
    "container": ["unknown_container", "invalid_container", 1],
    "environment": ["test", {1: "test"}, {"test": 1}],
    "local_conf": ["conf", [1]],
    "bblayers_conf": ["bb_conf", [1]],
    "sstate_mirror": [1, 1.4],
    "dl_dir": [1],
}


def test_validate(oebuild_repo_init_object):

    # case - similar with repo obj as it has repo instead of git_trees
    oebuild_obj = asdict(oebuild_repo_init_object)
    OEBuild.validate(oebuild_obj)
    # handling all defined test in test_map
    sources = oebuild_obj["sources"]
    for key in test_map:
        if key in ["repo", "git_trees"]:
            if key == "repo":
                oebuild_obj["sources"]["git_trees"] = None
            else:
                oebuild_obj["sources"]["repo"] = None
            for test_case in test_map[key]:
                oebuild_obj["sources"][key] = test_case
                with pytest.raises(TuxbakeParsingError):
                    OEBuild.validate(oebuild_obj)
            oebuild_obj["sources"] = sources

        else:
            prev_val = oebuild_obj[key]
            for test_case in test_map[key]:
                oebuild_obj[key] = test_case
                with pytest.raises(TuxbakeParsingError):
                    OEBuild.validate(oebuild_obj)
            oebuild_obj[key] = prev_val

    with pytest.raises(TuxbakeParsingError):
        # case - when both repo and git_trees are not present
        oebuild_obj["sources"]["repo"] = None
        oebuild_obj["sources"]["git_trees"] = None
        OEBuild.validate(oebuild_obj)


def test_as_dict(oebuild_repo_init_object):
    data = oebuild_repo_init_object.as_dict()
    assert isinstance(data, dict)


def test_prepare(oebuild_repo_init_object):
    oebuild_obj = oebuild_repo_init_object
    oebuild_obj.local_manifest = None
    oebuild_obj.prepare()
    assert oebuild_obj._runtime == "null"
    assert oebuild_obj._runtime.source_dir == Path(oebuild_obj.src_dir)
    assert oebuild_obj._runtime.basename == "build"
    assert oebuild_obj._runtime.environment["MACHINE"] == oebuild_obj.machine
    assert oebuild_obj._runtime.environment["DISTRO"] == oebuild_obj.distro

    # reading extra_local.conf file from src dir
    file = Path(f"{os.path.abspath(oebuild_obj.src_dir)}/extra_local.conf")
    if file.exists():
        with open(file, "r") as f:
            data = f.read()

            # validating conf
            if oebuild_obj.dl_dir:
                assert oebuild_obj.dl_dir in data
            if oebuild_obj.sstate_mirror:
                assert oebuild_obj.sstate_mirror in data
                assert 'USER_CLASSES += "buildstats buildstats-summary"\n' in data
            if oebuild_obj.local_conf:
                for val in oebuild_obj.local_conf:
                    assert val in data

    # reading bblayers.conf file from src dir
    file = Path(f"{os.path.abspath(oebuild_obj.src_dir)}/bblayers.conf")
    if file.exists():
        with open(file, "r") as f:
            data = f.read()

            # validating bblayers_conf
            if oebuild_obj.bblayers_conf:
                for val in oebuild_obj.bblayers_conf:
                    assert val in data



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from tuxbake.models import OEBuild
from tuxmake.runtime import Terminated
import signal


def build(**kwargs):
    old_sigterm = signal.signal(signal.SIGTERM, Terminated.handle_signal)
    oebuild = OEBuild(**kwargs)
    try:
        oebuild.prepare()
        oebuild.do_build()
    except (KeyboardInterrupt, Terminated) as ex:
        print("tuxbake Interrupted")
    oebuild.do_cleanup()
    signal.signal(signal.SIGTERM, old_sigterm)
    return oebuild



from unittest.mock import patch
import argparse


def test_setup_parser():
    from tuxbake.argparse import setup_parser

    assert isinstance(setup_parser(), argparse.ArgumentParser)

    """
      ( -- ) Refers to named optional arguments, i.e parser_map data can be in any order and also optional until specified as required.
    """
    parser_map = {
        "--build-definition": "oniro.json",
        "--runtime": "docker",
        "--image": None,
        "--src-dir": None,
        "--build-dir-name": "build",
        "--local-manifest": None,
        "--pinned-manifest": None,
    }
    data = ["test.py"]  # adding first argument as file_name
    for key in parser_map:
        data.extend([key, parser_map[key]])
    with patch("sys.argv", data):
        data = setup_parser().parse_args()
        assert all(
            [
                data.build_definition == parser_map["--build-definition"],
                data.runtime == parser_map["--runtime"],
                data.image == parser_map["--image"],
                data.src_dir == parser_map["--src-dir"],
                data.build_dir_name == parser_map["--build-dir-name"],
                data.local_manifest == parser_map["--local-manifest"],
                data.pinned_manifest == parser_map["--pinned-manifest"],
            ]
        )



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import argparse
from tuxmake.runtime import Runtime
from typing import List


class supported:
    runtimes: List[str] = Runtime.supported()


##########
# Setups #
##########
def setup_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(prog="tuxbake", description="TuxBake")
    #    parser.add_argument(
    #    "--version", action="version", version=f"%(prog)s, {__version__}"
    # )
    group = parser.add_argument_group("OE Build Parameters")
    group.add_argument(
        "--build-definition",
        help="Specify json file with build parameters",
        required=True,
    )
    group.add_argument(
        "--build-only",
        help="Assume the source tree is already pulled.",
        action="store_true",
    )
    group.add_argument(
        "--debug",
        help="Provides extra output on stderr for debugging tuxbake itself. This output will not appear in the build log.",
        action="store_true",
    )
    group.add_argument(
        "-r",
        "--runtime",
        help=f"Runtime to use for the builds. By default, builds are run in a docker container. With 'docker-local' runtime the --image option needs to be specified to a local container. Supported: {', '.join(supported.runtimes)}.",
        default="docker",
    )
    group.add_argument(
        "-i",
        "--image",
        help="Image to build with, for container-based runtimes (docker etc). Implies --runtime=docker if no runtime is explicit specified. (default: tuxbake-provided images).",
        default=None,
    )
    group.add_argument(
        "--src-dir",
        help="source directory where the sources will be downloaded",
    )
    group.add_argument(
        "--sync-only",
        help="Download all the meta repositories via repo or git only.",
        action="store_true",
    )
    group.add_argument(
        "--build-dir-name",
        help="Directory name passed to the source script during OE build. It defaults to build",
        default="build",
    )
    group.add_argument(
        "--local-manifest",
        help="Path to a local manifest file which will be used during repo sync. This input is ignored if sources used is git_trees in the build definition",
        default=None,
    )
    group.add_argument(
        "--pinned-manifest",
        help="Path to a local manifest file which will be used to override repo setup before doing repo sync. This input is ignored if sources used is git_trees in the build definition",
        default=None,
    )
    return parser



import os
import subprocess
import pytest


oebuild_git_params = {
    "sources": {
        "git_trees": [
            {"url": "http://git.yoctoproject.org/git/poky", "branch": "main"},
            {"url": "https://github.com/ndechesne/meta-qcom", "branch": "main"},
        ]
    },
    "src_dir": "test/src",
    "build_dir": "test/build",
    "container": "ubuntu-20.04",
    "envsetup": "poky/oe-init-build-env",
    "distro": "poky",
    "machine": "dragonboard-845c",
    "target": "core-image-minimal",
    "bblayers_conf": ['BBLAYERS += "../meta-qcom/"'],
    "environment": {},
    "local_manifest": "tests/unit/test.xml",
    "pinned_manifest": "",
    "sstate_mirror": "test_sstate",
    "dl_dir": "test_dl_dir",
    "runtime": "",
}


class Helpers:
    """Generic utility functions."""

    @staticmethod
    def setup_git(git_path):
        """
        Create a repo at git_path with two files and one commit.
        """
        files = [
            {
                "filename": "one",
                "contents": "The nice thing about standards is that you have so many to choose from.",
            },
            {"filename": "two", "contents": "☁️🐧☁️"},
        ]
        for f in files:
            with open(os.path.join(git_path, f["filename"]), "w") as h:
                h.write(f["contents"])
        subprocess.check_call(["git", "init"], cwd=git_path)
        subprocess.check_call(["git", "add", "."], cwd=git_path)
        subprocess.check_call(
            ["git", "config", "--local", "user.name", '"pytest ci"'], cwd=git_path
        )
        subprocess.check_call(
            ["git", "config", "--local", "user.email", '"test@email.com"'], cwd=git_path
        )
        subprocess.check_call(["git", "commit", "-m", '"Initial commit"'], cwd=git_path)

    @staticmethod
    def get_branch(path):
        """Get branch name from some repo at path"""
        return (
            subprocess.check_output(
                ["git", "rev-parse", "--abbrev-ref", "HEAD"], cwd=path
            )
            .strip()
            .decode("utf-8")
        )

    @staticmethod
    def get_sha(path):
        """Get first commit hash from some repo at path"""
        return (
            subprocess.check_output(
                ["git", "rev-list", "--max-parents=0", "HEAD"], cwd=path
            )
            .strip()
            .decode("utf-8")
        )


@pytest.fixture
def helpers():
    """Provide a helpers fixture so it's available to all tests."""
    return Helpers


@pytest.fixture()
def oebuild_git_object(helpers, tmpdir_factory):
    from tuxbake.models import OEBuild

    git_repo_1 = tmpdir_factory.mktemp("git_1")
    git_repo_2 = tmpdir_factory.mktemp("git_2")
    helpers.setup_git(git_repo_1)
    helpers.setup_git(git_repo_2)
    git_branch_1 = helpers.get_branch(git_repo_1)
    git_branch_2 = helpers.get_branch(git_repo_2)
    git_sha_1 = helpers.get_sha(git_repo_1)
    git_sha_2 = helpers.get_sha(git_repo_2)
    oebuild_git_params["sources"]["git_trees"][0]["url"] = f"{git_repo_1}"
    oebuild_git_params["sources"]["git_trees"][1]["url"] = f"{git_repo_2}"
    oebuild_git_params["sources"]["git_trees"][0]["branch"] = git_branch_1
    oebuild_git_params["sources"]["git_trees"][1]["branch"] = git_branch_2
    oebuild_git_params["sources"]["git_trees"][0]["sha"] = git_sha_1
    oebuild_git_params["sources"]["git_trees"][1]["sha"] = git_sha_2
    source_dir = tmpdir_factory.mktemp("src")
    oebuild_git_params["src_dir"] = source_dir

    oebuild = OEBuild(**oebuild_git_params)
    return (
        oebuild,
        git_repo_1,
        git_repo_2,
        git_branch_1,
        git_branch_2,
        git_sha_1,
        git_sha_2,
        source_dir,
    )


oebuild_repo_object = {
    "sources": {
        "repo": {
            "url": "https://gitlab.com/alok.ranjan1/test-project",
            "branch": "main",
            "manifest": "default.xml",
        },
    },
    "src_dir": "test/src",
    "build_dir": "test/build",
    "container": "ubuntu-20.04",
    "envsetup": "poky/oe-init-build-env",
    "distro": "poky",
    "machine": "dragonboard-845c",
    "target": "core-image-minimal",
    "local_conf": [
        'PREFERRED_PROVIDER_virtual/kernel = "linux-generic-mainline"',
        'PREFERRED_VERSION_linux-generic-mainline = "git%"',
        'SRCREV_kernel_hikey = "1d1df41c5a33359a00e919d54eaebfb789711fdc"',
        'SRCREV_kernel_juno = "99613159ad749543621da8238acf1a122880144e"',
        'SRCREV_ltp_hikey = "9b0740b72622f56c4d1909e2fdf38e948cf85b53"',
        'PREFERRED_VERSION_ltp = "20210927+git%"',
    ],
    "bblayers_conf": ['BBLAYERS += "../meta-qcom/"'],
    "environment": {"TEMPLATECONF": "../oniro/flavours/linux"},
    "local_manifest": "tests/unit/test.xml",
    "pinned_manifest": "",
    "sstate_mirror": "test_sstate",
    "dl_dir": "test_dl_dir",
    "runtime": "",
}


@pytest.fixture()
def oebuild_repo_init_object():
    from tuxbake.models import OEBuild

    oebuild = OEBuild(**oebuild_repo_object)
    return oebuild



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import json

from tuxbake.models import OEBuild
from tuxbake.argparse import setup_parser
from tuxbake.build import build
from tuxmake.runtime import Terminated
from tuxbake.exceptions import (
    TuxbakeParsingError,
    TuxbakeRunCmdError,
)

##############
# Entrypoint #
##############
def main() -> int:
    # Parse command line
    parser = setup_parser()
    options = parser.parse_args()
    with open(options.build_definition) as reader:
        try:
            build_definition = json.load(reader)
            OEBuild.validate(build_definition)
            build(
                **(build_definition),
                src_dir=options.src_dir,
                build_dir=options.build_dir_name,
                local_manifest=options.local_manifest,
                pinned_manifest=options.pinned_manifest,
                runtime=options.runtime,
                image=options.image,
                debug=options.debug,
                build_only=options.build_only,
                sync_only=options.sync_only,
            )
        except (TuxbakeParsingError, TuxbakeRunCmdError) as ex:
            sys.stderr.write(f"{str(ex)}\n")
            sys.exit(1)


def start():
    if __name__ == "__main__":
        sys.exit(main())


start()



class TuxbakeError(Exception):
    """Base class for all Tuxbuild exceptions"""

    error_help = ""
    error_type = ""


class TuxbakeRunCmdError(TuxbakeError):
    error_help = "Process call failed"
    error_type = "Configuration"


class TuxbakeParsingError(TuxbakeError):
    error_help = "Error while parsing API arguments"
    error_type = "Validation"


class TuxbakeValidationError(TuxbakeError):
    error_help = "Error while validating API arguments"
    error_type = "Validation"



import os
import subprocess
from tuxbake.exceptions import TuxbakeRunCmdError


def repo_init(oebuild, src_dir, local_manifest=None, pinned_manifest=None):
    cmd = f"repo init -u {oebuild.repo.url} -b {oebuild.repo.branch} -m {oebuild.repo.manifest}".split()
    run_cmd(cmd, src_dir)
    if pinned_manifest:
        cmd = f"cp {pinned_manifest} .repo/manifests/{oebuild.repo.manifest}".split()
        run_cmd(cmd, src_dir)

    if local_manifest:
        cmd = f"mkdir -p .repo/local_manifests/".split()
        run_cmd(cmd, src_dir)
        cmd = f"cp {local_manifest} .repo/local_manifests/".split()
        run_cmd(cmd, src_dir)
    cmd = "repo sync -j16".split()
    run_cmd(cmd, src_dir)
    cmd = "repo manifest -r -o pinned-manifest.xml".split()
    run_cmd(cmd, src_dir)


def git_init(oebuild, src_dir):
    for git_object in oebuild.git_trees:
        url = git_object.url
        branch = git_object.branch
        ref = git_object.ref
        sha = git_object.sha
        basename = os.path.splitext(os.path.basename(url))[0]
        if branch:
            cmd = f"git clone {url} -b {branch}".split()
        else:
            cmd = f"git clone {url}".split()
        run_cmd(cmd, src_dir)
        if ref:
            cmd = f"git fetch origin {ref}:{ref}-local".split()
            run_cmd(cmd, f"{src_dir}/{basename}")
            cmd = f"git checkout {ref}-local".split()
            run_cmd(cmd, f"{src_dir}/{basename}")
        if sha:
            cmd = f"git checkout {sha}".split()
            run_cmd(cmd, f"{src_dir}/{basename}")


def run_cmd(cmd, src_dir, env=None, fail_ok=False):
    process = subprocess.Popen(cmd, cwd=src_dir, env=env)
    process.communicate()
    if not fail_ok and process.returncode != 0:
        raise TuxbakeRunCmdError(f"Failed to run: {' '.join(cmd)}")



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['tuxbake']

package_data = \
{'': ['*']}

install_requires = \
['tuxmake >=1.2.0']

entry_points = \
{'console_scripts': ['tuxbake = tuxbake.__main__:main']}

setup(name='tuxbake',
      version='0.0.8',
      description='Command line tool to build OpenEmbedded and Yocto images',
      author='Vishal Bhoj',
      author_email='vishal.bhoj@linaro.org',
      url='https://tuxbake.org/',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
